using System.Collections.Generic;
using Microsoft.EntityFrameworkCore;
using InternshipSystem.Core;
using InternshipSystem.Repository.Model;

namespace InternshipSystem.Repository
{
    public class InternshipDbContext : DbContext
    {
        public DbSet<Document> Documents { get; set; }
        public DbSet<Course> Courses { get; set; }
        public DbSet<CompanyReadModel> Companies { get; set; }
        public DbSet<InternshipSubjectReadModel> Subjects { get; set; }
        public DbSet<EditionReadModel> Editions { get; set; }
        public DbSet<Student> Students { get; set; }
        public DbSet<Approval> Approvals { get; set; }
        public DbSet<BranchOfficeReadModel> BranchOffices { get; set; }
        public DbSet<InternshipRegistrationReadModel> Registrations { get; set; }
        public DbSet<InternshipProgramReadModel> Programs { get; set; }
        public DbSet<InternshipReadModel> Internships { get; set; }
        public DbSet<InternshipTypeReadModel> InternshipTypes { get; set; }

        public InternshipDbContext(DbContextOptions<InternshipDbContext> options)
            : base(options)
        {          
        }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder) =>
            optionsBuilder
                .UseSnakeCaseNamingConvention();

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<ProgramSubject>(builder =>
            {
                builder
                    .HasKey(subject => new { subject.InternshipProgramId, subject.InternshipSubjectId });
                
                builder
                    .HasOne<InternshipProgramReadModel>()
                    .WithMany(model => model.ChosenSubjects)
                    .HasForeignKey(subject => subject.InternshipProgramId);
                
                builder
                    .HasOne<InternshipSubjectReadModel>()
                    .WithMany()
                    .HasForeignKey(subject => subject.InternshipSubjectId);
            });
            
            modelBuilder.Entity<EditionSubject>(builder =>
            {
                builder
                    .HasKey(subject => new { subject.EditionId, subject.InternshipId});
                
                builder
                    .HasOne<EditionReadModel>()
                    .WithMany(model => model.AvailableSubjects)
                    .HasForeignKey(p => p.EditionId);

                builder
                    .HasOne<InternshipSubjectReadModel>()
                    .WithMany()
                    .HasForeignKey(subject => subject.InternshipId);
            });
        }
    }
}